@@ -2,8 +2,9 @@ from __future__ import annotations as _annotations
 
 import os
 import re
+from collections.abc import Callable
 from dataclasses import dataclass
-from typing import Callable, Literal, overload
+from typing import Literal, overload
 
 from pydantic_ai.exceptions import UserError
 from pydantic_ai.profiles import ModelProfile
@@ -27,15 +28,24 @@ except ImportError as _import_error:
     ) from _import_error
 
 
-@dataclass
+@dataclass(kw_only=True)
 class BedrockModelProfile(ModelProfile):
     """Profile for models used with BedrockModel.
 
     ALL FIELDS MUST BE `bedrock_` PREFIXED SO YOU CAN MERGE THEM WITH OTHER MODELS.
     """
 
-    bedrock_supports_tool_choice: bool = True
+    bedrock_supports_tool_choice: bool = False
     bedrock_tool_result_format: Literal['text', 'json'] = 'text'
+    bedrock_send_back_thinking_parts: bool = False
+
+
+def bedrock_amazon_model_profile(model_name: str) -> ModelProfile | None:
+    """Get the model profile for an Amazon model used via Bedrock."""
+    profile = amazon_model_profile(model_name)
+    if 'nova' in model_name:
+        return BedrockModelProfile(bedrock_supports_tool_choice=True).update(profile)
+    return profile
 
 
 class BedrockProvider(Provider[BaseClient]):
@@ -55,14 +65,14 @@ class BedrockProvider(Provider[BaseClient]):
 
     def model_profile(self, model_name: str) -> ModelProfile | None:
         provider_to_profile: dict[str, Callable[[str], ModelProfile | None]] = {
-            'anthropic': lambda model_name: BedrockModelProfile(bedrock_supports_tool_choice=False).update(
-                anthropic_model_profile(model_name)
-            ),
+            'anthropic': lambda model_name: BedrockModelProfile(
+                bedrock_supports_tool_choice=True, bedrock_send_back_thinking_parts=True
+            ).update(anthropic_model_profile(model_name)),
             'mistral': lambda model_name: BedrockModelProfile(bedrock_tool_result_format='json').update(
                 mistral_model_profile(model_name)
             ),
             'cohere': cohere_model_profile,
-            'amazon': amazon_model_profile,
+            'amazon': bedrock_amazon_model_profile,
             'meta': meta_model_profile,
             'deepseek': deepseek_model_profile,
         }
