@@ -3,6 +3,7 @@ from __future__ import annotations as _annotations
 import os
 
 import httpx
+from typing_extensions import deprecated
 
 from pydantic_ai.exceptions import UserError
 from pydantic_ai.models import cached_async_http_client
@@ -11,6 +12,7 @@ from pydantic_ai.profiles.google import google_model_profile
 from pydantic_ai.providers import Provider
 
 
+@deprecated('`GoogleGLAProvider` is deprecated, use `GoogleProvider` with `GoogleModel` instead.')
 class GoogleGLAProvider(Provider[httpx.AsyncClient]):
     """Provider for Google Generative Language AI API."""
 
